# These libraries are required to run the code.  They can be loaded
# with the command:  install.packages(repos="http://bioinformatics.mdanderson.org/OOMPA")

library(ClassComparison)
library(ClassDiscovery)

# Source the file with the necessary functions
source("VSFunction.R")


############################################
# Load the data
#load(file="Z:/MyPapers/VSnorm/thedata.Rda")
load(file="thedata.Rda")

# This RData object contains 4 items:
# 1.  mydata: the matrix of raw quantifications.  This has 1152 rows (because the set of
#			  arrays had 1152 dilution series) and 138 columns for the 138 proteins.
#			  Most of the dilution series were run in duplicate.  There are 563 biological
#			  replicates in this set.
# 2.  rep1: A numerical vector of length 563 with the indices corresponding to the replicates
#			on the left side of the array
# 3.  rep2: A numerical vector of length 563 with the indices corresponding to the replicates
#			on the right side of the arrays
# 4.  all.list: A logical vector of length 563 that is true if the sample is from ALL disease
#				and false otherwise.


##############################################
# Normalization Methods:
#
# Housekeeping

mydata.house <- sweep(mydata,1,mydata[,"House"],"-")

#
# Median
rowmedianB <- apply(mydata,1,median,na.rm=T)
colmedianB <- apply(mydata,2,median,na.rm=T)
temp <- sweep(mydata,2,colmedianB)
mydata.med <- sweep(temp,1,rowmedianB)
mydata.med <- sweep(mydata,1,rowmedianB)

#
# Variable Slope
colMedianB <- apply(mydata,2,median,na.rm=T)
rowMedianB <- apply(mydata,1,median,na.rm=T)
xij <- sweep(mydata,2,colMedianB)
gammaB <- estimateGamma(xij,method="other")
mydata.gam <- sweep(xij,2,gammaB,"/")
mydata.gam <- sweep(mydata.gam,1,rowMedianB,"-")


############################################
############################################
#Figure 2
par(mfrow=c(2,2))

#par(mfrow=c(1,4))
par(mar=(c(4,4,3,0)+.01))

pch1 <- 17
col1 <- 'gray60'
pch2 <- 20
col2 <- 'gray2'
xx1 <- .8
xx2 <- .6

a <- mydata[361,]
b <- mydata[718,]
mi <- min(c(a,b))
ma <- max(c(a,b))
plot(a,pch=pch1,col=col1,ylim=c(mi,ma),xlab="Array Index", ylab="Log Concentration",cex.axis=xx2,cex.lab=xx2,cex=xx)
points(b,pch=pch2,col=col2,cex=xx)
abline(h=median(a),col=col1)
abline(h=median(b),col=col2)
title("No Normalization",cex.main=.9)

par(mar=(c(4,3,3,1)+.01))

a <- mydata.house[361,]
b <- mydata.house[718,]
mi <- min(c(a,b))
ma <- max(c(a,b))
plot(a,pch=pch1,col=col1,ylim=c(mi,ma),xlab="Array Index", ylab="Log Concentration",cex.axis=xx2,cex.lab=xx2,cex=xx)
points(b,pch=pch2,col=col2,cex=xx)
abline(h=median(a),col=col1)
abline(h=median(b),col=col2)
title("Housekeeping",cex.main=.9)

a <- mydata.med[361,]
b <- mydata.med[718,]
mi <- min(c(a,b))
ma <- max(c(a,b))
plot(a,pch=pch1,col=col1,ylim=c(mi,ma),xlab="Array Index", ylab="Log Concentration",cex.axis=xx2,cex.lab=xx2,cex=xx)
points(b,pch=pch2,col=col2,cex=xx)
abline(h=median(a),col=col1)
abline(h=median(b),col=col2)
title("Median",cex.main=.9)

a <- mydata.gam[361,]
b <- mydata.gam[718,]
mi <- min(c(a,b))
ma <- max(c(a,b))
plot(a,pch=pch1,col=col1,ylim=c(mi,ma),xlab="Array Index", ylab="Log Concentration",cex.axis=xx2,cex.lab=xx2,cex=xx)
points(b,pch=pch2,col=col2,cex=xx)
abline(h=median(a),col=col1)
abline(h=median(b),col=col2)
title("Variable Slope",cex.main=.9)


#####################
######################################
# Take averages of replicates and extract the ALL samples
# None

temp <- matrix(NA,nrow=length(rep1),ncol=ncol(mydata))
dimnames(temp)[[2]] <- dimnames(mydata)[[2]]
for (i in 1:ncol(mydata)) {
	conc <- mydata[,i]
	c1.rep1 <- conc[rep1]
	c1.rep2 <- conc[rep2]
	temp[,i] <- apply(cbind(c1.rep1,c1.rep2),1,mean,na.rm=T)
}
allNone <- temp[all.list,]

#
# House

temp <- matrix(NA,nrow=length(rep1),ncol=ncol(mydata))
dimnames(temp)[[2]] <- dimnames(mydata)[[2]]
for (i in 1:ncol(mydata)) {
	conc <- mydata.house[,i]
	c1.rep1 <- conc[rep1]
	c1.rep2 <- conc[rep2]
	temp[,i] <- apply(cbind(c1.rep1,c1.rep2),1,mean,na.rm=T)
}
allHouse <- temp[all.list,]


#
# Median
#
temp <- matrix(NA,nrow=length(rep1),ncol=ncol(mydata))
dimnames(temp)[[2]] <- dimnames(mydata)[[2]]
for (i in 1:ncol(mydata)) {
	conc <- mydata.med[,i]
	c1.rep1 <- conc[rep1]
	c1.rep2 <- conc[rep2]
	temp[,i] <- apply(cbind(c1.rep1,c1.rep2),1,mean,na.rm=T)
}
allMedian <- temp[all.list,]

#
# Variable Slope


temp <- matrix(NA,nrow=length(rep1),ncol=ncol(mydata))
dimnames(temp)[[2]] <- dimnames(mydata)[[2]]
for (i in 1:ncol(mydata)) {
	conc <- mydata.gam[,i]
	c1.rep1 <- conc[rep1]
	c1.rep2 <- conc[rep2]
	temp[,i] <- apply(cbind(c1.rep1,c1.rep2),1,mean,na.rm=T)
}
allGamma <- temp[all.list,]

###########################################
# To reproduce Bootstrap clusters

gps <- cutHclust(allGamma, 9, method = "average", metric = "pearson")
cols <- rep("cornflowerblue",length(gps))
cols[gps==2] <- "navy"
cols[gps==3] <- "orange"
cols[gps==4] <- "goldenrod4"
cols[gps==5] <- "red"
cols[gps==6] <- "green"
cols[gps==7] <- "purple3"
cols[gps==8] <- "firebrick"
cols[gps==9] <- "cyan"

gps <- cutHclust(allGamma[,-54], 9, method = "average", metric = "pearson")
cols2 <- rep("firebrick",length(gps))
cols2[gps==2] <- "springgreen2"
cols2[gps==3] <- "purple"
cols2[gps==4] <- "goldenrod4"
cols2[gps==5] <- "darkgray"
cols2[gps==6] <- "gold"
cols2[gps==7] <- "hotpink"
cols2[gps==8] <- "cornflowerblue"
cols2[gps==9] <- "cyan"


k <- 9
bcMed <- BootstrapClusterTest(allMedian, cutHclust, nTimes=300, k=k, metric="pearson",method="average",verbose=F)
bcVS <- BootstrapClusterTest(allGamma, cutHclust, nTimes=300, k=k, metric="pearson",method="average",verbose=F)

image(bcMed,col=blueyellow(64),RowSideColors=cols,ColSideColors=cols,labRow="",labCol="")
title(sub='Median',cex.sub=3.5)

image(bcVS,col=blueyellow(64),RowSideColors=cols,ColSideColors=cols,labRow="",labCol="")
title(sub='Variable Slope',cex.sub=3.5)



##################################
# Get the percent inconsistent 

# Median
temp1 <- mydata.med[rep1,]
temp2 <- mydata.med[rep2,]
allMed1 <- temp1[all.list,]
allMed2 <- temp2[all.list,]

# Variable Slope
temp1 <- mydata.gam[rep1,]
temp2 <- mydata.gam[rep2,]
allGam1 <- temp1[all.list,]
allGam2 <- temp2[all.list,]

psMed <- pairscore(allMed1,allMed2,k=9)
psGam <- pairscore(allGam1,allGam2,k=9)

psMed
psGam


