require(MASS)
require(ClassDiscovery)
source("VSFunction.R")


######################################
# Do simulations of different scenarios
nSim <- 500

params <- matrix(c(30,200,4,2,.1, 70,200,4,2,.1, 10,200,4,2,.1, 30,75,4,2,.1, 30,800,4,2,.1, 30,200,2,2,.1, 
  30,200,10,2,.1, 30, 200, 1000, 2, .1, 30,200,4,1,.1, 30,200,4,5,.1, 30,200,4,2,.05, 30,200,4,2,.5,
  30,200,4,2,0,20,200,4,4,.1),ncol=5,byrow=T)
dimnames(params)[[2]] <- c("nCol","nRow","rowSD","colSD","slopeSD")

mseMed <- mseGam <- matrix(0,ncol=6,nrow=nrow(params))
mseOne <- meanOne <- varOne <- mseMed
gE <- matrix(0,ncol=2,nrow=nrow(params))
                 
#for (i2 in 1:nrow(params)) {
for (i2 in 6:6){
simMed <- simGam <- simOne <- matrix(0,nrow=nSim,ncol=6)
gammaEst <- rep(0,nSim)

for(i1 in 1:nSim){
	
	nRow <- params[i2,"nRow"]
	nCol <- params[i2, "nCol"]
	rowSD <- params[i2, "rowSD"]
	colSD <- params[i2, "colSD"]
	slopeSD <- params[i2, "slopeSD"]

	# Get the TRUTH MATRIX!!!	
	Truth <- 1*matrix(rnorm(nRow*nCol),nrow=nRow,ncol=nCol)
	Truth[,2:3] <- mvrnorm(nRow,mu=c(0,0),Sigma=matrix(c(1,.6,.6,1),ncol=2,nrow=2)) 
	Truth[,6:7] <- mvrnorm(nRow,mu=c(0,0),Sigma=matrix(c(1,-.4,-.4,1),ncol=2,nrow=2)) 
	rowEffects <- rowSD*rnorm(nRow)-2
	colEffects <- colSD*rnorm(nCol)-1
	X <- sweep(Truth,1,rowEffects,"+")
	X[,nCol] <- rowEffects + rnorm(nRow,0,.5)
	X <- sweep(X,2,colEffects,"+")
	X[1,4] <- X[1,4] + 10
	X[2,4] <- X[2,4] + 5
	colJitters <- exp(slopeSD*rnorm(nCol))
	X <- sweep(X,2,colJitters,"*")
	
	Xhat <- X - mean(X)
	
	rowMedian <- apply(Xhat,1,median)
	colMedian <- apply(Xhat,2,median)
	
	#median
	Xhatmed <- sweep(Xhat,1,rowMedian,"-")
	Xhatmed <- sweep(Xhatmed,2,colMedian,"-")
	
	#one protein
	Xhatone <- sweep(Xhat,1,Xhat[,nCol],"-")
	Xhatone <- sweep(Xhatone,2,colMedian,"-")
	
	#gamma
	Xhatgam <- sweep(Xhat,2,colMedian)
	gamma <- estimateGamma(Xhatgam)
	Xhatgam <- sweep(Xhatgam,2,gamma,"/")
	Xhatgam <- sweep(Xhatgam,1,rowMedian,"-")
	
	simMed[i1,1] <- cor(Xhatmed[,2],Xhatmed[,3])
	simGam[i1,1] <- cor(Xhatgam[,2],Xhatgam[,3])
	simOne[i1,1] <- cor(Xhatone[,2],Xhatone[,3])
	
	simMed[i1,2] <- cor(Xhatmed[,3],Xhatmed[,8])
	simGam[i1,2] <- cor(Xhatgam[,3],Xhatgam[,8])
	simOne[i1,2] <- cor(Xhatone[,3],Xhatone[,8])
	
	simMed[i1,3] <- cor(Xhatmed[,6],Xhatmed[,7])
	simGam[i1,3] <- cor(Xhatgam[,6],Xhatgam[,7])
	simOne[i1,3] <- cor(Xhatone[,6],Xhatone[,7])

	simMed[i1,4] <- Xhatmed[1,4] - Xhatmed[3,4]
	simGam[i1,4] <- Xhatgam[1,4] - Xhatgam[3,4]
	simOne[i1,4] <- Xhatone[1,4] - Xhatone[3,4]

	simMed[i1,5] <- Xhatmed[2,4] - Xhatmed[3,4]
	simGam[i1,5] <- Xhatgam[2,4] - Xhatgam[3,4]
	simOne[i1,5] <- Xhatone[2,4] - Xhatone[3,4]
	
	simMed[i1,6] <- Xhatmed[2,4] - Xhatmed[2,5]
	simGam[i1,6] <- Xhatgam[2,4] - Xhatgam[2,5]
	simOne[i1,6] <- Xhatone[2,4] - Xhatone[2,5]
	
	gammaEst[i1] <- sum((gamma-colJitters)^2)
}

#mse
mseMed[i2,1] <- var(simMed[,1]) + (.6-mean(simMed[,1]))^2
mseMed[i2,2] <- var(simMed[,2]) + (0-mean(simMed[,2]))^2
mseMed[i2,3] <- var(simMed[,3]) + (-0.4-mean(simMed[,3]))^2
mseMed[i2,4] <- var(simMed[,4]) + (10-mean(simMed[,4]))^2
mseMed[i2,5] <- var(simMed[,5]) + (5-mean(simMed[,5]))^2
mseMed[i2,6] <- var(simMed[,6]) + (5-mean(simMed[,6]))^2

mseGam[i2,1] <- var(simGam[,1]) + (.6-mean(simGam[,1]))^2
mseGam[i2,2] <- var(simGam[,2]) + (0-mean(simGam[,2]))^2
mseGam[i2,3] <- var(simGam[,3]) + (-0.4-mean(simGam[,3]))^2
mseGam[i2,4] <- var(simGam[,4]) + (10-mean(simGam[,4]))^2
mseGam[i2,5] <- var(simGam[,5]) + (5-mean(simGam[,5]))^2
mseGam[i2,6] <- var(simGam[,6]) + (5-mean(simGam[,6]))^2

mseOne[i2,1] <- var(simOne[,1]) + (.6-mean(simOne[,1]))^2
mseOne[i2,2] <- var(simOne[,2]) + (0-mean(simOne[,2]))^2
mseOne[i2,3] <- var(simOne[,3]) + (-0.4-mean(simOne[,3]))^2
mseOne[i2,4] <- var(simOne[,4]) + (10-mean(simOne[,4]))^2
mseOne[i2,5] <- var(simOne[,5]) + (5-mean(simOne[,5]))^2
mseOne[i2,6] <- var(simOne[,6]) + (5-mean(simOne[,6]))^2


gE[i2,1] <- mean(gammaEst)
gE[i2,2] <- var(gammaEst) 
}

a <- cbind(params,mseOne[,1],mseMed[,1],mseGam[,1],mseOne[,4],mseMed[,4],mseGam[,4])
xtable(a,digits=c(0,0,0,0,0,2,3,3,3,2,2,2))

b <- cbind(params,gE[,1])
xtable(b,digits=c(0,0,0,0,0,2,3))



#################################################
#################################################
pcaPlot <- function(pca,cols,pchs,groups=groups,myc=1.8,...) {
	plot(pca@scores[,1],pca@scores[,2],type='n',...)
    ngroups <- length(unique(groups))
	if(missing(pchs)) pchs <- 1:ngroups
	if(missing(cols)) cols <- 1:ngroups
	for (i in 1:5) {
		mycex <- myc
		if(pchs[i]==8) mycex <- 1
		points(pca@scores[groups==i,1],pca@scores[groups==i,2],col=cols[i],pch=pchs[i],cex=mycex)
		#axis(1,labels=F)
		#axis(2,labels=F)
		box()
	}
}
###############################
# Run simulations with 5 known groups of proteins

runCluster <- F
if(runCluster==T) {

g1 <- 3
g2 <- 10
g3 <- 5
g4 <- 5
g5 <- 7
nCol <- g1+g2+g3+g4+g5

sigma <- diag(nCol)
s1 <- matrix(.4,ncol=g1,nrow=g1)
s2 <- matrix(.2,ncol=g2,nrow=g2)
s3 <- matrix(.2,ncol=g3,nrow=g3)
s4 <- matrix(.5,ncol=g4,nrow=g4)
s5 <- matrix(.3,ncol=g5,nrow=g5)
sigma[1:3,1:3] <- s1
sigma[4:13,4:13] <- s2
sigma[14:18,14:18] <- s3
sigma[19:23,19:23] <- s4
sigma[24:30,24:30] <- s5
diag(sigma) <- rep(1,nCol)

nRow <- 200
groups <- as.factor(c(rep(1,g1),rep(2,g2),rep(3,g3),rep(4,g4),rep(5,g5)))
set.seed(46241)
Truth <- mvrnorm(nRow,mu=rep(-2,nCol),Sigma=sigma) 

nRow <- nrow(Truth)
nCol <- ncol(Truth)
#rowSD <- 5
#colSD <- 2
#slopeSD <- .2

rowSD <- 4.3
colSD <- 2
slopeSD <- .1

rowEffects <- rowSD*rnorm(nRow)-2
colEffects <- colSD*rnorm(nCol)-1
X <- sweep(Truth,1,rowEffects,"+")
X <- sweep(X,2,colEffects,"+")
colJitters <- exp(slopeSD*rnorm(nCol))
X <- sweep(X,2,colJitters,"*")
	
Xhat <- X - mean(X)
rowMedian <- apply(Xhat,1,median)
colMedian <- apply(Xhat,2,median)
	
#median
Xhatmed <- sweep(Xhat,1,rowMedian,"-")
Xhatmed <- sweep(Xhatmed,2,colMedian,"-")
	
#gamma
Xhatgam <- sweep(Xhat,2,colMedian)
gamma <- estimateGamma(Xhatgam)
Xhatgam <- sweep(Xhatgam,2,gamma,"/")
Xhatgam <- sweep(Xhatgam,1,rowMedian,"-")


######################################	
## Make plot of the principal components 
mycol <- c("black","gray26","gray42","gray58","gray74")
pca <- SamplePCA(Truth,splitter=groups)
pca0 <- SamplePCA(X,splitter=groups)
pca1 <- SamplePCA(Xhatmed,splitter=groups)
pca2 <- SamplePCA(Xhatgam,splitter=groups)


par(mfrow=c(2,2))
par(mar=(c(4,4,3,1)+.01))
pcaPlot(pca,cols=mycol,pch=c(15,8,16,17,18),groups=groups,myc=1.5,xlab='Component 1',ylab='Component 2');title("Truth")
pcaPlot(pca0,cols=mycol,pch=c(15,8,16,17,18),groups=groups,myc=1.5,xlab='Component 1',ylab='Component 2');title("Observed")
pcaPlot(pca1,cols=mycol,pch=c(15,8,16,17,18),groups=groups,myc=1.5,xlab='Component 1',ylab='Component 2');title("Median Loading")
pcaPlot(pca2,cols=mycol,pch=c(15,8,16,17,18),groups=groups,myc=1.5,xlab='Component 1',ylab='Component 2');title("Variable Slope")


##############################
#  clustering and bootstrap clusters

cols <- c(rep("cornflowerblue",g1),rep("firebrick",g2),rep("goldenrod",g3),rep("darkorchid",g4),rep("seagreen4",g5))
labs <- c(rep("1",g1),rep("2",g2),rep("3",g3),rep("4",g4),rep("5",g5))


bc1 <- BootstrapClusterTest(Xhatmed, cutHclust, nTimes=500, k=5, metric='pearson',verbose=F)
bc2 <- BootstrapClusterTest(Xhatgam, cutHclust, nTimes=500, k=5, metric='pearson',verbose=F)


hcmed <- hclust(distanceMatrix(Xhatmed, 'pearson'), method='average')
hcgam <- hclust(distanceMatrix(Xhatgam, 'pearson'), method='average')

#Clusters for Median Normalization
plotColoredClusters(hcmed,cols=cols,labs=labs)
title("Median Loading")
image(bc1,col=blueyellow(128),RowSideColors=cols,ColSideColors=cols)
title("Median Loading")

#Clusters for VS Normalization

plotColoredClusters(hcgam,cols=cols,labs=labs)
title("Variable Slope")
image(bc2,col=blueyellow(128),RowSideColors=cols,ColSideColors=cols)
title("Variable Slope")

}




